package dmc;

/*
 * TabbedConnection.java
 * Matthew Wronka <wronkm@rpi.edu>
 *
 * Tabbed dialog for all active connections.
 * Each connection contains TabbedChat.
 *
 * Copyright (C) 2002 Matthew Wronka <wronkm@rpi.edu>
 * This file is released under the GNU General Public License version 2,
 * a version of which should have been provided along with this file.
 *
 * You might also find a copy at:
 * http://dmc.sf.net/files/gpl.txt
 * http://www.fsf.org/copyleft/gpl.html
 */


import javax.swing.*;
import javax.swing.text.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

import java.net.*;
import java.io.IOException;

import wtalk.WTalk;


/**
 * Manages multiple connections, comprised of TabbedChats.
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 0
 */
public class TabbedConnection{
	final protected JTabbedPane tabbedPane = new JTabbedPane();
	protected wtalk.IUserList ul = null;


	/**
	 * Default ctor. 
	 */
	public TabbedConnection() { initTabbedConnection("usa"); }
	public TabbedConnection(String sProtocol) { 
		initTabbedConnection(sProtocol); 
	}

	/**
	 * Initializes the TabbedConnection
	 *
	 * @param sProtocol Protocol to create an instance of.
	 */
	public void initTabbedConnection(String sProtocol) {
		tabbedPane.setTabPlacement(JTabbedPane.TOP);
		tabbedPane.setFocusable(false);
		addListeners();
		addKeyBindings();

		openConnection("DMC", sProtocol);
	}



	/**
	 * Sets the object to register with WTalkProtocols that will
	 * implement IUserList to support GUI representation of buddy lists.
	 *
	 * @param u Object that will be used to manage user lists.
	 */
	public void setUserList(wtalk.IUserList u) { 
		ul = u; 
	}

	/**
	 * Adds keybindigns to facilitate switching connections.
	 */
	private void addKeyBindings() {
		tabbedPane.getInputMap(
			JComponent.WHEN_IN_FOCUSED_WINDOW).
			put(KeyStroke.getKeyStroke("ctrl J"), "next-connection");
		tabbedPane.getInputMap(
			JComponent.WHEN_IN_FOCUSED_WINDOW).
			put(KeyStroke.getKeyStroke("ctrl K"), "prev-connection");

		Action NextConnection = new AbstractAction(){
			public void actionPerformed(ActionEvent e){	
				int i=tabbedPane.getSelectedIndex();
				if(i==tabbedPane.getTabCount()-1)
					tabbedPane.setSelectedIndex(0);
				else
					tabbedPane.setSelectedIndex(i+1);
			}
		};

		Action PrevConnection = new AbstractAction(){
			public void actionPerformed(ActionEvent e){
				int i=tabbedPane.getSelectedIndex();
				if(i==0)
					tabbedPane.setSelectedIndex(
						tabbedPane.getTabCount()-1);
				else
					tabbedPane.setSelectedIndex(i-1);

			}
		};


		tabbedPane.getActionMap().put("next-connection", NextConnection);
		tabbedPane.getActionMap().put("prev-connection", PrevConnection);
	}


	/**
	 * Adds event listeners to tabs to facilitate highlighting of 
	 * tabs which do not have focus when they receive messages.
	 */
	private void addListeners() {
		tabbedPane.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e) {
				tabbedPane.setForegroundAt(tabbedPane.getSelectedIndex(), DmcColors.TAB_NORMAL);
			}
		});
	}

	/**
	 * Returns the GUI representation of a TabbedChat.
	 *
	 * @return Returns GUI  representation of this object.
	 */
	public JTabbedPane getUI() {
		return tabbedPane;
	}

	/**
	 * Returns the index of the tab with caption 's' in tabbedPane.
	 *
	 * @param s Caption of target tag.
	 * @return Index of tag with Caption 's', or -1 on failure.
	 */
	public int indexOfTab(String s) {
		for(int i = tabbedPane.getTabCount(); --i >= 0 ;) {
			if(s.equalsIgnoreCase(tabbedPane.getTitleAt(i)))
				return i;
		}
		return -1;
	}


	/**
	 * Adds a new tab to the TabbedConnection with caption <em>name</em>.
	 *	
 	 * @param name Caption for tab. 
	 * @param type Protocol to be used by this TabbedChat.
	 *	These are defined in wtalk.Protocol
	 * @return TabbedChat
	 * @see TabbedChat
	 * @see wtalk.Protocol
	 */
	private TabbedChat addTab(String name, int type) {
		TabbedChat tc = new TabbedChat(this, name);
		tc.getUI().setTabPlacement(JTabbedPane.BOTTOM);

		tc.setConnectionType(type);
		//tc.setConnName(name);
		//tc.setConnManager(this);
		tc.setUserList(ul);
		
		tabbedPane.addTab(name, tc.getUI());
		tabbedPane.setForegroundAt(tabbedPane.indexOfTab(name), DmcColors.TAB_NORMAL);

		return tc;
	}

	/**
	 * Returns the TabbedChat displaying the connection 
	 *	with <em>name</em>.  If none exists, a new tab is added
	 *	which contains a TabbdChat to use in communications with
	 *	this buddy.
	 *
	 * @param name The requested connection's TabbedChat which
	 *	is to be returned.
	 * @param protocol Name of the requested protocol.
	 * @return TabbedChat used to represent the given connection
	 *	 with <em>name</em>.
	 */
	public TabbedChat openConnection(String name, String protocol) {
		int i = getUI().indexOfTab(name);
		TabbedChat tc=null;
		if(i>=0) 
			//tc = ((TabbedChat)(getUI().getComponentAt(i))); 
			;
		else
			tc = addTab(name, wtalk.Protocol.getProtocolByName(protocol));

		return tc;
	}

	/**
	 * Returns the TabbedChat of the connection; does not create one.
	 *
	 * @param name Name of the Connection
	 * @return TabbedChat used to represent the given connection.
	 */
	public TabbedChat getTabbedChat(String name) {
		int i = getUI().indexOfTab(name);
		TabbedChat tc=null;
		if(i>=0) {
			tc = ((JChatPanel)
				((JTabbedPane)getUI().getComponentAt(i))
				.getSelectedComponent()).getTabbedChat();

		}

		return tc;
	}
}
