package wtalk.protocols;

/*
 * Copyright (C) 2002 Matthew Wronka <wronkm@rpi.edu>
 * This file is released under the GNU General Public License version 2,
 * a version of which should have been provided along with this file.
 *
 * You might also find a copy at:
 * http://dmc.sf.net/files/gpl.txt
 * http://www.fsf.org/copyleft/gpl.html
 */

import wtalk.RMsgHandler;
import wtalk.IUserList;
import java.io.IOException;

/**
 * Generic interface for protocols supported by WTalk.
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 0
 */

public interface WTalkProtocol {

	//////////////////////////////////////////////////////////
	//
	// Callbacks
	//

	//
	// These are for writing messages ...
	//
	/**
	 * Sets default message handler.
	 * 
	 * @param Class which implements wtalk.RMsgHandler
	 * @see wtalk.RMsgHandler
	 */
	public void setReceivedMessageHandler(RMsgHandler r);	// Default

	/**
	 * Sets message handler for chat mesages. 
	 * 
	 * @param Class which implements wtalk.RMsgHandler
	 * @see wtalk.RMsgHandler
	 */
	public void setChatMessageHandler(RMsgHandler r);	// Chats

	/**
	 * Sets message handler for private messages.
	 * 
	 * @param Class which implements wtalk.RMsgHandler
	 * @see wtalk.RMsgHandler
	 */
	public void setPrivateMessageHandler(RMsgHandler r);	// Private

	/**
	 * Sets message handler for global brodcasts (walls).
	 * 
	 * @param Class which implements wtalk.RMsgHandler
	 * @see wtalk.RMsgHandler
	 */
	public void setWallMessageHandler(RMsgHandler r);	// Walls

	/**
	 * Sets message handler for system messages.
	 * 
	 * @param Class which implements wtalk.RMsgHandler
	 * @see wtalk.RMsgHandler
	 */
	public void setSystemMessageHandler(RMsgHandler r);	// System..

	/**
	 * Sets handler for automated messages.
	 * 
	 * @param Class which implements wtalk.RMsgHandler
	 * @see wtalk.RMsgHandler
	 */
	public void setAutoMessageHandler(RMsgHandler r);	// Auto..

	//
	// Sets call back for manipulating buddy-lists.
	//
	/**
	 * WTalk requires some file to handle lists of users,
	 * 	known as "Buddy Lists".  This class must
	 *	implement IUserList, though it need not actually
	 *	do anything with the the information provided.
	 *
	 * @param ul Class which manages User Lists.
	 * @see wtalk.IUserList
	 */
	public void setUserListHandler(IUserList ul);
	//
	//////////////////////////////////////////////////////////


	//////////////////////////////////////////////////////////
	//
	// Uncooked functions
	//
	/**
	 * Sends an "unformatted" or "raw" message.
	 * <p>
	 * What this means on a protocol-to-protocol basis
	 *	varies, though for text-based protocols
	 *	it is generally the message by itself.
	 *
	 * @param A string to send as a "raw" message.
	 */
	public void sendRaw(String msg); 	// sends literal string
	//public void interpret(String cmd);	// Interpret a command
	//
	//////////////////////////////////////////////////////////


	//////////////////////////////////////////////////////////
	//
	// Authentication/Login
	//
	/**
	 * Connects to a preset server and port.
	 *
	 * @throws IOException Throws excetption if it cannot connect 
	 *	to the host; for example, invalid hostname or 
	 *	no service running.
	 */
	public void connect() throws IOException; 	// includes sign-on

	/**
	 * Overloaded to set user name to 'login', and then connects 
	 *	to a preset server and port.
	 * 
	 * @throws IOException Throws excetption if it cannot connect 
	 *	to the host; for example, invalid hostname or 
	 *	no service running.
	 */
	public void connect(String login) throws IOException;

	/**
	 * Overloaded to connects to a specified server and port--and 
	 *	remembers these values so that connect() will use them.
	 *
	 * @throws IOException Throws excetption if it cannot connect 
	 *	to the host; for example, invalid hostname or 
	 *	no service running.
	 */
	public void connect(String server, int port) throws IOException;

	/**
	 * Overloaded to set user name to 'login' and then connects 
	 *	to a specified server and port--and remembers these 
	 *	values so that connect() will use them.
	 *
	 * @throws IOException Throws excetption if it cannot connect 
	 *	to the host; for example, invalid hostname or 
	 *	no service running.
	 */
	public void connect(String server, int port, String login) throws IOException;

	/**
	 * Disconnects if the WTalkProtocol is connected.
	 */
	public void disconnect(String msg); 	// includes sign-off

	/**
	 * Returns whether the WTalkProtocol object has an active connection. 
	 * 
	 * @return Returns true if the WTalkProtocol has an active connection.
	 */
	public boolean isConnected();	// Are we currently connected?


	/**
	 * Sets the user's name for the active connection to <em>name</em>
	 *
	 * @param name User name to request.  Depending on the protocol
	 *	it is not guaranteed that this request will be granted,
	 *	or if it will even make since once connected.
	 */
	public void setName(String name); 	// sets handle

	/**
	 * Returns the user's name (handle) 
	 *	understood by the current connection.
	 *
	 * @return The user's current handle on this connection.
	 */
	public String getName();

	/**
	 * Sets the password the connection will use to authenticate.
	 * <p>
	 * Note: this is not the same as 
	 *	<a href="#changePassword"><code>changePassword</code></a>.
	 *
	 * @param passwd Password to use to authenticate.
	 */
	public void setPassword(String passwd); 

	/**
	 * Changes the password that will be used for future authentication
	 * 	attempts for the user.
	 *
	 * @param oldpwd The old password, such as the one used to authenticate
	 *	with when loggin on.
	 * @param newpwd A new password, to be used in future authentication
	 * 	attempts.
	 */
	public void changePassword(String oldpwd, String newpwd); 

	/**
	 * Sets the server to use when <code>connect()</code> is called.
	 * <p>
	 * This function <strong>SHOULD NOT</strong> attempt to connect.
	 *
	 * @param server Host that <code>connect()</code> will connect to.
	 */
	public void setServer(String server);	// sets server -- SHOULD NOT connect

	/**
	 * Returns the name of the current server that <code>connect()</code>
	 * 	will use.  This is often the server we are connected to
	 *	but may not be if <code>setServer()</code> has been called
	 *	since we have connected.
	 *
	 * @return Name of current server WTalkProtocol has stored.
	 */
	public String getServer();	// returns name of current server

	/**
	 * Sets the port to use when connecting to a host.
	 *
	 * @param port Port number to use when connecting. 
	 * 	<p>
	 *	If port==0, then the protocol should use a "Default" port.
	 */
	public void setServerPort(int port);

	/**
	 * Returns current Port; the last port set by setServerPort().
	 * Note that this may return 0, indicating the default port
	 * is being used.
	 *
	 * @return The current Port stored by the WTalkProtocol.
	 */
	public int getServerPort();
	//
	//////////////////////////////////////////////////////////


	//////////////////////////////////////////////////////////
	//
	// Communication functions
	//

	//
	// IM functions
	//
	/**
	 * Sends a "normal" text message, data.
	 *
	 * @param to Target of message.
	 * @param data Message contents.
	 */
	public void sendMessage(String to, String data); 

	/**
	 * Requests information on "who is &lt;<em>user</em>&gt;".
	 *
	 * @param user Name of user to requst information on.
	 */
	public void whois(String user);

	/**
	 * Adds a buddy to a "Buddy List" stored or managed by
	 *	either ther server or WTalk, as opposed to 
	 *	the shell.
	 *
	 * @param buddy Buddy's name to be added.
	 * @param group Group to which Buddy will be added.
	 */
	public void addBuddy(String buddy, String group);

	/**
	 * Removes a buddy from a "Buddy List" stored or managed by
	 *	either the server or WTalk, as opposed to
	 *	the shell.
	 *
	 * @param buddy Buddy's which will be removed.
	 * @param group Group from which Buddy will be removed.
	 */
	public void removeBuddy(String buddy, String group);
	//listBuddies();


	/**
	 * Sets away status and messages, or removes away status.
	 *
	 * @param message Sets <em>message</em> as away message. 
	 *	if <em>message</em> is null, away status is removed.
	 */
	public void setAway(String message);

	//addDeny();
	//removeDeny();


	//
	// Chat functions
	//
	/**
	 * Joins chat <em>room</em>.
	 *
	 * @param room Name of chat to join.
	 */
	public void joinChat(String room);

	/**
	 * Leaves (parts) chat <em>room</em>.
	 *
	 * @param room Name of chat to part.
	 */
	public void partChat(String room); 


	/**
	 * Sends a message to all members in <em>room</em>.
	 *
	 * @param room The room to which a message is sent.
	 * @param data The contents of the message.
	 */
	public void chatMessage(String room, String data);

	/**
	 * Sets or retrieves the topic of <em>room</em>.
	 *
	 * @param room Room whose topic we are concerned.
	 * @param topic If not null, the topic which will be
	 *	set in <em>room</em>.  If null, a request
	 * 	to receive the current topic is sent.
	 */
	public void setTopic(String room, String topic); 

	/**
	 * Requests information on a chat, <em>room</em>, or all
	 * 	chats on the server [system].
	 *
	 * @param room The chat which we would like information.
	 *	A null value indicates a request for all active chats
	 *	if this is permitted by the protocol (it may not be).
	 */
	public void chatList(String room);	// room may be null!


	//
	// Misc. messaging
	//
	/**
	 * Sends a brodcast (wall) message to all users.
	 *
	 * @param msg The message to brodcast.
	 */
	public void sendWall(String msg);
	//
	//////////////////////////////////////////////////////////



	//////////////////////////////////////////////////////////
	//
	// Admin functions
	//
	/**
	 * Sets or removes a ban on <em>user</em>.
	 *
	 * @param user The user with whose bannedness we are concerned.
	 * @param banned A true value bans <em>user</em>, false unbans <em>uer</em>.
	 */
	public void setBan(String user, boolean banned);
	//
	//////////////////////////////////////////////////////////
}
